from .__init__ import ctypes, typing, abc, \
    SDL_POINTER, SDL_FUNC, SDL_BINARY

from .SDL_stdinc import SDL_FunctionPointer

class SDL_SharedObject(ctypes.c_void_p):
    ...

SDL_LoadObject: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LoadObject", SDL_POINTER[SDL_SharedObject], [ctypes.c_char_p], SDL_BINARY]
SDL_LoadFunction: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_LoadFunction", SDL_FunctionPointer, [SDL_POINTER[SDL_SharedObject], ctypes.c_char_p], SDL_BINARY]
SDL_UnloadObject: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UnloadObject", None, [SDL_POINTER[SDL_SharedObject]], SDL_BINARY]


from .__init__ import ctypes, typing, abc, SDL_PLATFORM_SPECIFIC, \
    SDL_POINTER, SDL_FUNC, SDL_FUNC_TYPE, SDL_BINARY

from .SDL_init import SDL_AppInit_func, \
    SDL_AppIterate_func, SDL_AppEvent_func, SDL_AppQuit_func

SDL_main_func: typing.TypeAlias = SDL_FUNC_TYPE["SDL_main_func", ctypes.c_int, [ctypes.c_int, SDL_POINTER[ctypes.c_char_p]]]

SDL_SetMainReady: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetMainReady", None, [], SDL_BINARY]

SDL_RunApp: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RunApp", ctypes.c_int, [ctypes.c_int, SDL_POINTER[ctypes.c_char_p], SDL_main_func, ctypes.c_void_p], SDL_BINARY]
SDL_EnterAppMainCallbacks: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_EnterAppMainCallbacks", ctypes.c_int, [ctypes.c_int, SDL_POINTER[ctypes.c_char_p], SDL_AppInit_func, SDL_AppIterate_func, SDL_AppEvent_func, SDL_AppQuit_func], SDL_BINARY]

if SDL_PLATFORM_SPECIFIC(system = ["Windows"]):
    SDL_RegisterApp: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RegisterApp", ctypes.c_bool, [ctypes.c_char_p, ctypes.c_uint32, ctypes.c_void_p], SDL_BINARY]
    SDL_UnregisterApp: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UnregisterApp", None, [], SDL_BINARY]


from .__init__ import ctypes, typing, abc, \
    SDL_POINTER, SDL_ENUM, SDL_FUNC, SDL_TYPE, SDL_BINARY

from .SDL_properties import SDL_PropertiesID

class SDL_Sensor(ctypes.c_void_p):
    ...

SDL_SensorID: typing.TypeAlias = SDL_TYPE["SDL_SensorID", ctypes.c_uint32]

SDL_STANDARD_GRAVITY: float = 9.80665

SDL_SensorType: typing.TypeAlias = SDL_TYPE["SDL_SensorType", SDL_ENUM]

SDL_SENSOR_INVALID, SDL_SENSOR_UNKNOWN, SDL_SENSOR_ACCEL, SDL_SENSOR_GYRO, \
    SDL_SENSOR_ACCEL_L, SDL_SENSOR_GYRO_L, SDL_SENSOR_ACCEL_R, SDL_SENSOR_GYRO_R = range(-1, 7)

SDL_GetSensors: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensors", SDL_POINTER[SDL_SensorID], [SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_GetSensorNameForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensorNameForID", ctypes.c_char_p, [SDL_SensorID], SDL_BINARY]
SDL_GetSensorTypeForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensorTypeForID", SDL_SensorType, [SDL_SensorID], SDL_BINARY]
SDL_GetSensorNonPortableTypeForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensorNonPortableTypeForID", ctypes.c_int, [SDL_SensorID], SDL_BINARY]
SDL_OpenSensor: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_OpenSensor", SDL_POINTER[SDL_Sensor], [SDL_SensorID], SDL_BINARY]
SDL_GetSensorFromID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensorFromID", SDL_POINTER[SDL_Sensor], [SDL_SensorID], SDL_BINARY]
SDL_GetSensorProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensorProperties", SDL_PropertiesID, [SDL_POINTER[SDL_Sensor]], SDL_BINARY]
SDL_GetSensorName: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensorName", ctypes.c_char_p, [SDL_POINTER[SDL_Sensor]], SDL_BINARY]
SDL_GetSensorType: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensorType", SDL_SensorType, [SDL_POINTER[SDL_Sensor]], SDL_BINARY]
SDL_GetSensorNonPortableType: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensorNonPortableType", ctypes.c_int, [SDL_POINTER[SDL_Sensor]], SDL_BINARY]
SDL_GetSensorID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensorID", SDL_SensorID, [SDL_POINTER[SDL_Sensor]], SDL_BINARY]
SDL_GetSensorData: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSensorData", ctypes.c_bool, [SDL_POINTER[SDL_Sensor], SDL_POINTER[ctypes.c_float], ctypes.c_int], SDL_BINARY]
SDL_CloseSensor: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CloseSensor", None, [SDL_POINTER[SDL_Sensor]], SDL_BINARY]
SDL_UpdateSensors: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UpdateSensors", None, [], SDL_BINARY]


from .__init__ import ctypes, typing, abc, \
    SDL_POINTER, SDL_ENUM, SDL_FUNC, SDL_TYPE, SDL_BINARY

from .SDL_properties import SDL_PropertiesID
from .SDL_keycode import SDL_Keycode, SDL_Keymod
from .SDL_scancode import SDL_Scancode
from .SDL_video import SDL_Window
from .SDL_rect import SDL_Rect

SDL_KeyboardID: typing.TypeAlias = SDL_TYPE["SDL_KeyboardID", ctypes.c_uint32]

SDL_HasKeyboard: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasKeyboard", ctypes.c_bool, [], SDL_BINARY]
SDL_GetKeyboards: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetKeyboards", SDL_POINTER[SDL_KeyboardID], [SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_GetKeyboardNameForID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetKeyboardNameForID", ctypes.c_char_p, [SDL_KeyboardID], SDL_BINARY]
SDL_GetKeyboardFocus: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetKeyboardFocus", SDL_POINTER[SDL_Window], [], SDL_BINARY]
SDL_GetKeyboardState: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetKeyboardState", SDL_POINTER[ctypes.c_bool], [SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_ResetKeyboard: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ResetKeyboard", None, [], SDL_BINARY]
SDL_GetModState: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetModState", SDL_Keymod, [], SDL_BINARY]
SDL_SetModState: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetModState", None, [SDL_Keymod], SDL_BINARY]
SDL_GetKeyFromScancode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetKeyFromScancode", SDL_Keycode, [SDL_Scancode, SDL_Keymod, ctypes.c_bool], SDL_BINARY]
SDL_GetScancodeFromKey: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetScancodeFromKey", SDL_Scancode, [SDL_Keycode, SDL_POINTER[SDL_Keymod]], SDL_BINARY]
SDL_SetScancodeName: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetScancodeName", ctypes.c_bool, [SDL_Scancode, ctypes.c_char_p], SDL_BINARY]
SDL_GetScancodeName: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetScancodeName", ctypes.c_char_p, [SDL_Scancode], SDL_BINARY]
SDL_GetScancodeFromName: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetScancodeFromName", SDL_Scancode, [ctypes.c_char_p], SDL_BINARY]
SDL_GetKeyName: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetKeyName", ctypes.c_char_p, [SDL_Keycode], SDL_BINARY]
SDL_GetKeyFromName: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetKeyFromName", SDL_Keycode, [ctypes.c_char_p], SDL_BINARY]

SDL_StartTextInput: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_StartTextInput", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_TextInputType: typing.TypeAlias = SDL_TYPE["SDL_TextInputType", SDL_ENUM]

SDL_TEXTINPUT_TYPE_TEXT, SDL_TEXTINPUT_TYPE_TEXT_NAME, SDL_TEXTINPUT_TYPE_TEXT_EMAIL, SDL_TEXTINPUT_TYPE_TEXT_USERNAME, \
    SDL_TEXTINPUT_TYPE_TEXT_PASSWORD_HIDDEN, SDL_TEXTINPUT_TYPE_TEXT_PASSWORD_VISIBLE, SDL_TEXTINPUT_TYPE_NUMBER, \
        SDL_TEXTINPUT_TYPE_NUMBER_PASSWORD_HIDDEN, SDL_TEXTINPUT_TYPE_NUMBER_PASSWORD_VISIBLE = range(9)

SDL_Capitalization: typing.TypeAlias = SDL_TYPE["SDL_Capitalization", SDL_ENUM]

SDL_CAPITALIZE_NONE, SDL_CAPITALIZE_SENTENCES, SDL_CAPITALIZE_WORDS, SDL_CAPITALIZE_LETTERS = range(4)

SDL_StartTextInputWithProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_StartTextInputWithProperties", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_PropertiesID], SDL_BINARY]

SDL_PROP_TEXTINPUT_TYPE_NUMBER: bytes = "SDL.textinput.type".encode()
SDL_PROP_TEXTINPUT_CAPITALIZATION_NUMBER: bytes = "SDL.textinput.capitalization".encode()
SDL_PROP_TEXTINPUT_AUTOCORRECT_BOOLEAN: bytes = "SDL.textinput.autocorrect".encode()
SDL_PROP_TEXTINPUT_MULTILINE_BOOLEAN: bytes = "SDL.textinput.multiline".encode()
SDL_PROP_TEXTINPUT_ANDROID_INPUTTYPE_NUMBER: bytes = "SDL.textinput.android.inputtype".encode()

SDL_TextInputActive: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_TextInputActive", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_StopTextInput: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_StopTextInput", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_ClearComposition: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ClearComposition", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_SetTextInputArea: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetTextInputArea", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[SDL_Rect], ctypes.c_int], SDL_BINARY]
SDL_GetTextInputArea: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetTextInputArea", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[SDL_Rect], SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_HasScreenKeyboardSupport: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HasScreenKeyboardSupport", ctypes.c_bool, [], SDL_BINARY]
SDL_ScreenKeyboardShown: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ScreenKeyboardShown", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]


from .__init__ import typing, SDL_TYPE, SDL_ENUM

SDL_Scancode: typing.TypeAlias = SDL_TYPE["SDL_Scancode", SDL_ENUM]

SDL_SCANCODE_UNKNOWN: int = 0

SDL_SCANCODE_A, SDL_SCANCODE_B, SDL_SCANCODE_C, SDL_SCANCODE_D, SDL_SCANCODE_E, SDL_SCANCODE_F, SDL_SCANCODE_G, SDL_SCANCODE_H, SDL_SCANCODE_I, SDL_SCANCODE_J, \
    SDL_SCANCODE_K, SDL_SCANCODE_L, SDL_SCANCODE_M, SDL_SCANCODE_N, SDL_SCANCODE_O, SDL_SCANCODE_P, SDL_SCANCODE_Q, SDL_SCANCODE_R, SDL_SCANCODE_S, SDL_SCANCODE_T, \
        SDL_SCANCODE_U, SDL_SCANCODE_V, SDL_SCANCODE_W, SDL_SCANCODE_X, SDL_SCANCODE_Y, SDL_SCANCODE_Z, \
    \
    SDL_SCANCODE_1, SDL_SCANCODE_2, SDL_SCANCODE_3, SDL_SCANCODE_4, SDL_SCANCODE_5, SDL_SCANCODE_6, SDL_SCANCODE_7, SDL_SCANCODE_8, SDL_SCANCODE_9, SDL_SCANCODE_0, \
    \
    SDL_SCANCODE_RETURN, SDL_SCANCODE_ESCAPE, SDL_SCANCODE_BACKSPACE, SDL_SCANCODE_TAB, SDL_SCANCODE_SPACE, \
    \
    SDL_SCANCODE_MINUS, SDL_SCANCODE_EQUALS, SDL_SCANCODE_LEFTBRACKET, SDL_SCANCODE_RIGHTBRACKET, SDL_SCANCODE_BACKSLASH, SDL_SCANCODE_NONUSHASH, \
        SDL_SCANCODE_SEMICOLON, SDL_SCANCODE_APOSTROPHE, SDL_SCANCODE_GRAVE, SDL_SCANCODE_COMMA, SDL_SCANCODE_PERIOD, SDL_SCANCODE_SLASH, SDL_SCANCODE_CAPSLOCK, \
    \
    SDL_SCANCODE_F1, SDL_SCANCODE_F2, SDL_SCANCODE_F3, SDL_SCANCODE_F4, SDL_SCANCODE_F5, SDL_SCANCODE_F6, \
        SDL_SCANCODE_F7, SDL_SCANCODE_F8, SDL_SCANCODE_F9, SDL_SCANCODE_F10, SDL_SCANCODE_F11, SDL_SCANCODE_F12, \
    \
    SDL_SCANCODE_PRINTSCREEN, SDL_SCANCODE_SCROLLLOCK, SDL_SCANCODE_PAUSE, SDL_SCANCODE_INSERT, SDL_SCANCODE_HOME, SDL_SCANCODE_PAGEUP, SDL_SCANCODE_DELETE, \
        SDL_SCANCODE_END, SDL_SCANCODE_PAGEDOWN, SDL_SCANCODE_RIGHT, SDL_SCANCODE_LEFT, SDL_SCANCODE_DOWN, SDL_SCANCODE_UP, \
    \
    SDL_SCANCODE_NUMLOCKCLEAR, SDL_SCANCODE_KP_DIVIDE, SDL_SCANCODE_KP_MULTIPLY, SDL_SCANCODE_KP_MINUS, SDL_SCANCODE_KP_PLUS, SDL_SCANCODE_KP_ENTER, \
        SDL_SCANCODE_KP_1, SDL_SCANCODE_KP_2, SDL_SCANCODE_KP_3, SDL_SCANCODE_KP_4, SDL_SCANCODE_KP_5, SDL_SCANCODE_KP_6, SDL_SCANCODE_KP_7, SDL_SCANCODE_KP_8, \
            SDL_SCANCODE_KP_9, SDL_SCANCODE_KP_0, SDL_SCANCODE_KP_PERIOD, \
    \
    SDL_SCANCODE_NONUSBACKSLASH, SDL_SCANCODE_APPLICATION, SDL_SCANCODE_POWER, SDL_SCANCODE_KP_EQUALS, SDL_SCANCODE_F13, SDL_SCANCODE_F14, SDL_SCANCODE_F15, \
        SDL_SCANCODE_F16, SDL_SCANCODE_F17, SDL_SCANCODE_F18, SDL_SCANCODE_F19, SDL_SCANCODE_F20, SDL_SCANCODE_F21, SDL_SCANCODE_F22, SDL_SCANCODE_F23, SDL_SCANCODE_F24, \
            SDL_SCANCODE_EXECUTE, SDL_SCANCODE_HELP, SDL_SCANCODE_MENU, SDL_SCANCODE_SELECT, SDL_SCANCODE_STOP, SDL_SCANCODE_AGAIN, SDL_SCANCODE_UNDO, SDL_SCANCODE_CUT, \
                SDL_SCANCODE_COPY, SDL_SCANCODE_PASTE, SDL_SCANCODE_FIND, SDL_SCANCODE_MUTE, SDL_SCANCODE_VOLUMEUP, SDL_SCANCODE_VOLUMEDOWN, *_, \
    \
    SDL_SCANCODE_KP_COMMA, SDL_SCANCODE_KP_EQUALSAS400, \
        SDL_SCANCODE_INTERNATIONAL1, SDL_SCANCODE_INTERNATIONAL2, SDL_SCANCODE_INTERNATIONAL3, SDL_SCANCODE_INTERNATIONAL4, SDL_SCANCODE_INTERNATIONAL5, SDL_SCANCODE_INTERNATIONAL6, \
            SDL_SCANCODE_INTERNATIONAL7, SDL_SCANCODE_INTERNATIONAL8, SDL_SCANCODE_INTERNATIONAL9, SDL_SCANCODE_LANG1, SDL_SCANCODE_LANG2, SDL_SCANCODE_LANG3, SDL_SCANCODE_LANG4, \
                SDL_SCANCODE_LANG5, SDL_SCANCODE_LANG6, SDL_SCANCODE_LANG7, SDL_SCANCODE_LANG8, SDL_SCANCODE_LANG9, \
    \
    SDL_SCANCODE_ALTERASE, SDL_SCANCODE_SYSREQ, SDL_SCANCODE_CANCEL, SDL_SCANCODE_CLEAR, SDL_SCANCODE_PRIOR, SDL_SCANCODE_RETURN2, SDL_SCANCODE_SEPARATOR, \
        SDL_SCANCODE_OUT, SDL_SCANCODE_OPER, SDL_SCANCODE_CLEARAGAIN, SDL_SCANCODE_CRSEL, SDL_SCANCODE_EXSEL = range(4, 165)

SDL_SCANCODE_KP_00, SDL_SCANCODE_KP_000, SDL_SCANCODE_THOUSANDSSEPARATOR, SDL_SCANCODE_DECIMALSEPARATOR, SDL_SCANCODE_CURRENCYUNIT, SDL_SCANCODE_CURRENCYSUBUNIT, \
    SDL_SCANCODE_KP_LEFTPAREN, SDL_SCANCODE_KP_RIGHTPAREN, SDL_SCANCODE_KP_LEFTBRACE, SDL_SCANCODE_KP_RIGHTBRACE, SDL_SCANCODE_KP_TAB, SDL_SCANCODE_KP_BACKSPACE, SDL_SCANCODE_KP_A, \
        SDL_SCANCODE_KP_B, SDL_SCANCODE_KP_C, SDL_SCANCODE_KP_D, SDL_SCANCODE_KP_E, SDL_SCANCODE_KP_F, SDL_SCANCODE_KP_XOR, SDL_SCANCODE_KP_POWER, SDL_SCANCODE_KP_PERCENT, SDL_SCANCODE_KP_LESS, \
            SDL_SCANCODE_KP_GREATER, SDL_SCANCODE_KP_AMPERSAND, SDL_SCANCODE_KP_DBLAMPERSAND, SDL_SCANCODE_KP_VERTICALBAR, SDL_SCANCODE_KP_DBLVERTICALBAR, SDL_SCANCODE_KP_COLON, SDL_SCANCODE_KP_HASH, \
                SDL_SCANCODE_KP_SPACE, SDL_SCANCODE_KP_AT, SDL_SCANCODE_KP_EXCLAM, SDL_SCANCODE_KP_MEMSTORE, SDL_SCANCODE_KP_MEMRECALL, SDL_SCANCODE_KP_MEMCLEAR, SDL_SCANCODE_KP_MEMADD, SDL_SCANCODE_KP_MEMSUBTRACT, \
                    SDL_SCANCODE_KP_MEMMULTIPLY, SDL_SCANCODE_KP_MEMDIVIDE, SDL_SCANCODE_KP_PLUSMINUS, SDL_SCANCODE_KP_CLEAR, SDL_SCANCODE_KP_CLEARENTRY, SDL_SCANCODE_KP_BINARY, \
                        SDL_SCANCODE_KP_OCTAL, SDL_SCANCODE_KP_DECIMAL, SDL_SCANCODE_KP_HEXADECIMAL, *_, \
    \
    SDL_SCANCODE_LCTRL, SDL_SCANCODE_LSHIFT, SDL_SCANCODE_LALT, SDL_SCANCODE_LGUI, SDL_SCANCODE_RCTRL, \
        SDL_SCANCODE_RSHIFT, SDL_SCANCODE_RALT, SDL_SCANCODE_RGUI = range(176, 232)

SDL_SCANCODE_MODE, SDL_SCANCODE_SLEEP, SDL_SCANCODE_WAKE, SDL_SCANCODE_CHANNEL_INCREMENT, SDL_SCANCODE_CHANNEL_DECREMENT, \
    SDL_SCANCODE_MEDIA_PLAY, SDL_SCANCODE_MEDIA_PAUSE, SDL_SCANCODE_MEDIA_RECORD, SDL_SCANCODE_MEDIA_FAST_FORWARD, SDL_SCANCODE_MEDIA_REWIND, SDL_SCANCODE_MEDIA_NEXT_TRACK, \
        SDL_SCANCODE_MEDIA_PREVIOUS_TRACK, SDL_SCANCODE_MEDIA_STOP, SDL_SCANCODE_MEDIA_EJECT, SDL_SCANCODE_MEDIA_PLAY_PAUSE, SDL_SCANCODE_MEDIA_SELECT, \
    \
    SDL_SCANCODE_AC_NEW, SDL_SCANCODE_AC_OPEN, SDL_SCANCODE_AC_CLOSE, SDL_SCANCODE_AC_EXIT, \
        SDL_SCANCODE_AC_SAVE, SDL_SCANCODE_AC_PRINT, SDL_SCANCODE_AC_PROPERTIES, \
    \
    SDL_SCANCODE_AC_SEARCH, SDL_SCANCODE_AC_HOME, SDL_SCANCODE_AC_BACK, SDL_SCANCODE_AC_FORWARD, \
        SDL_SCANCODE_AC_STOP, SDL_SCANCODE_AC_REFRESH, SDL_SCANCODE_AC_BOOKMARKS, \
    \
    SDL_SCANCODE_SOFTLEFT, SDL_SCANCODE_SOFTRIGHT, \
        SDL_SCANCODE_CALL, SDL_SCANCODE_ENDCALL = range(257, 291)

SDL_SCANCODE_RESERVED, SDL_SCANCODE_COUNT = 400, 512


import os, sys, collections.abc as abc

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

import sdl3, ctypes, atexit, traceback

functions: dict[abc.Callable[..., None], list[str]] = {}

def TEST_RegisterFunction(systems: list[str]) -> abc.Callable[[abc.Callable[..., None]], None]:
    return lambda func: functions.update({func: systems})

from tests.TEST_init import *
from tests.TEST_locale import *
from tests.TEST_version import *
from tests.TEST_video import *

@atexit.register
def TEST_RunAllTests() -> None:
    if not functions: return
    print("\33[32m", f"Initializing tests... (version: {sdl3.__version__}, system: {sdl3.SDL_SYSTEM}, arch: {sdl3.SDL_ARCH}).", "\33[0m", sep = "", flush = True)
    passed, failed = 0, 0

    for func, systems in functions.items():
        try:
            if sdl3.SDL_SYSTEM not in systems:
                print("\33[33m", f"Test '{func.__name__}' is not supported on {sdl3.SDL_SYSTEM}.", "\33[0m", sep = "", flush = True)

            else:
                sdl3.SDL_ClearError(); func()
                print("\33[32m", f"Test '{func.__name__}' passed.", "\33[0m", sep = "", flush = True)
                passed += 1

        except AssertionError as exc:
            print("\33[31m", f"Test '{func.__name__}' failed: {str(exc).capitalize()}", "\33[0m", sep = "", flush = True)
            failed += 1
        
        except Exception as exc:
            print("\33[31m", f"Test '{func.__name__}' failed: {str(exc).capitalize()}.\n\n{traceback.format_exc()}", "\33[0m", sep = "", flush = True)
            os._exit(-1)

    print("\33[31m" if failed else "\33[32m", f"{'Failed' if failed else 'Passed'}! {passed} test(s) passed, {failed} test(s) failed.", "\33[0m", sep = "", flush = True)
    if failed: os._exit(-1)


import os, sys

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "temp")))
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

os.environ["SDL_DOC_GENERATOR"] = "0"
os.environ["SDL_CTYPES_ALIAS_FIX"] = "1"
os.environ["SDL_DISABLE_METADATA"] = "1"
os.environ["SDL_PLATFORM_AGNOSTIC"] = "1"
os.environ["SDL_IGNORE_MISSING_FUNCTIONS"] = "1"
os.environ["SDL_CHECK_BINARY_VERSION"] = "0"
os.environ["SDL_DOWNLOAD_BINARIES"] = "0"
os.environ["SDL_FIND_BINARIES"] = "0"
os.makedirs("temp", exist_ok = True)

import sdl3

for i in list(sdl3.SDL_BINARY_VAR_MAP_INV.keys()):
    with open(f"temp/{i}.py", "w") as file:
        file.write(sdl3.SDL_GENERATE_DOCS([i], rst = True))

project = "PySDL3"
copyright = "2025, Yusuf Rençber"
author = "Yusuf Rençber"
release = sdl3.__version__

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.inheritance_diagram",
    "sphinx.ext.intersphinx",
    "sphinx.ext.viewcode",
    "sphinx.ext.napoleon",
    "sphinx.ext.todo"
]

templates_path = []
exclude_patterns = ["build"]

language = "en"
pygments_style = None

html_theme = "sphinx_rtd_theme"
html_static_path = []


from .__init__ import ctypes, typing, abc, SDL_POINTER, \
    SDL_FUNC_TYPE, SDL_FUNC, SDL_TYPE, SDL_BINARY, SDL_ENUM

from .SDL_properties import SDL_PropertiesID
from .SDL_surface import SDL_Surface
from .SDL_pixels import SDL_PixelFormat
from .SDL_rect import SDL_Point, SDL_Rect
from .SDL_stdinc import SDL_FunctionPointer

SDL_DisplayID: typing.TypeAlias = SDL_TYPE["SDL_DisplayID", ctypes.c_uint32]
SDL_WindowID: typing.TypeAlias = SDL_TYPE["SDL_WindowID", ctypes.c_uint32]

SDL_PROP_GLOBAL_VIDEO_WAYLAND_WL_DISPLAY_POINTER: bytes = "SDL.video.wayland.wl_display".encode()

SDL_SystemTheme: typing.TypeAlias = SDL_TYPE["SDL_SystemTheme", SDL_ENUM]

SDL_SYSTEM_THEME_UNKNOWN, SDL_SYSTEM_THEME_LIGHT, SDL_SYSTEM_THEME_DARK = range(3)

class SDL_DisplayModeData(ctypes.c_void_p):
    ...

class SDL_DisplayMode(ctypes.Structure):
    _fields_ = [
        ("displayID", SDL_DisplayID),
        ("format", SDL_PixelFormat),
        ("w", ctypes.c_int),
        ("h", ctypes.c_int),
        ("pixel_density", ctypes.c_float),
        ("refresh_rate", ctypes.c_float),
        ("refresh_rate_numerator", ctypes.c_int),
        ("refresh_rate_denominator", ctypes.c_int),
        ("internal", SDL_POINTER[SDL_DisplayModeData])
    ]

SDL_DisplayOrientation: typing.TypeAlias = SDL_TYPE["SDL_DisplayOrientation", SDL_ENUM]

SDL_ORIENTATION_UNKNOWN, SDL_ORIENTATION_LANDSCAPE, SDL_ORIENTATION_LANDSCAPE_FLIPPED, \
    SDL_ORIENTATION_PORTRAIT, SDL_ORIENTATION_PORTRAIT_FLIPPED = range(5)

class SDL_Window(ctypes.c_void_p):
    ...

SDL_WindowFlags: typing.TypeAlias = SDL_TYPE["SDL_WindowFlags", ctypes.c_uint64]

SDL_WINDOW_FULLSCREEN: int = 0x0000000000000001
SDL_WINDOW_OPENGL: int = 0x0000000000000002
SDL_WINDOW_OCCLUDED: int = 0x0000000000000004
SDL_WINDOW_HIDDEN: int = 0x0000000000000008
SDL_WINDOW_BORDERLESS: int = 0x0000000000000010
SDL_WINDOW_RESIZABLE: int = 0x0000000000000020
SDL_WINDOW_MINIMIZED: int = 0x0000000000000040
SDL_WINDOW_MAXIMIZED: int = 0x0000000000000080
SDL_WINDOW_MOUSE_GRABBED: int = 0x0000000000000100
SDL_WINDOW_INPUT_FOCUS: int = 0x0000000000000200
SDL_WINDOW_MOUSE_FOCUS: int = 0x0000000000000400
SDL_WINDOW_EXTERNAL: int = 0x0000000000000800
SDL_WINDOW_MODAL: int = 0x0000000000001000
SDL_WINDOW_HIGH_PIXEL_DENSITY: int = 0x0000000000002000
SDL_WINDOW_MOUSE_CAPTURE: int = 0x0000000000004000
SDL_WINDOW_MOUSE_RELATIVE_MODE: int = 0x0000000000008000
SDL_WINDOW_ALWAYS_ON_TOP: int = 0x0000000000010000
SDL_WINDOW_UTILITY: int = 0x0000000000020000
SDL_WINDOW_TOOLTIP: int = 0x0000000000040000
SDL_WINDOW_POPUP_MENU: int = 0x0000000000080000
SDL_WINDOW_KEYBOARD_GRABBED: int = 0x0000000000100000
SDL_WINDOW_VULKAN: int = 0x0000000010000000
SDL_WINDOW_METAL: int = 0x0000000020000000
SDL_WINDOW_TRANSPARENT: int = 0x0000000040000000
SDL_WINDOW_NOT_FOCUSABLE: int = 0x0000000080000000

SDL_WINDOWPOS_UNDEFINED_MASK: int = 0x1FFF0000
SDL_WINDOWPOS_UNDEFINED_DISPLAY: abc.Callable[[SDL_DisplayID], int] = lambda x: SDL_WINDOWPOS_UNDEFINED_MASK | x
SDL_WINDOWPOS_UNDEFINED = SDL_WINDOWPOS_UNDEFINED_DISPLAY(0)
SDL_WINDOWPOS_ISUNDEFINED: abc.Callable[[int], bool] = lambda x: (x & 0xFFFF0000) == SDL_WINDOWPOS_UNDEFINED_MASK

SDL_WINDOWPOS_CENTERED_MASK: int = 0x2FFF0000
SDL_WINDOWPOS_CENTERED_DISPLAY: abc.Callable[[SDL_DisplayID], int] = lambda x: SDL_WINDOWPOS_CENTERED_MASK | x
SDL_WINDOWPOS_CENTERED = SDL_WINDOWPOS_CENTERED_DISPLAY(0)
SDL_WINDOWPOS_ISCENTERED: abc.Callable[[int], bool] = lambda x: (x & 0xFFFF0000) == SDL_WINDOWPOS_CENTERED_MASK

SDL_FlashOperation: typing.TypeAlias = SDL_TYPE["SDL_FlashOperation", SDL_ENUM]

SDL_FLASH_CANCEL, SDL_FLASH_BRIEFLY, SDL_FLASH_UNTIL_FOCUSED = range(3)

class SDL_GLContextState(ctypes.c_void_p):
    ...

SDL_GLContext: typing.TypeAlias = SDL_TYPE["SDL_GLContext", SDL_POINTER[SDL_GLContextState]]

SDL_EGLDisplay: typing.TypeAlias = SDL_TYPE["SDL_EGLDisplay", ctypes.c_void_p]
SDL_EGLConfig: typing.TypeAlias = SDL_TYPE["SDL_EGLConfig", ctypes.c_void_p]
SDL_EGLSurface: typing.TypeAlias = SDL_TYPE["SDL_EGLSurface", ctypes.c_void_p]
SDL_EGLAttrib: typing.TypeAlias = SDL_TYPE["SDL_EGLAttrib", ctypes.c_longlong]
SDL_EGLint: typing.TypeAlias = SDL_TYPE["SDL_EGLint", ctypes.c_int]

SDL_EGLAttribArrayCallback: typing.TypeAlias = SDL_FUNC_TYPE["SDL_EGLAttribArrayCallback", SDL_POINTER[SDL_EGLAttrib], [ctypes.c_void_p]]
SDL_EGLIntArrayCallback: typing.TypeAlias = SDL_FUNC_TYPE["SDL_EGLIntArrayCallback", SDL_POINTER[SDL_EGLint], [ctypes.c_void_p, SDL_EGLDisplay, SDL_EGLConfig]]

SDL_GLAttr: typing.TypeAlias = SDL_TYPE["SDL_GLAttr", SDL_ENUM]

SDL_GL_RED_SIZE, SDL_GL_GREEN_SIZE, SDL_GL_BLUE_SIZE, SDL_GL_ALPHA_SIZE, SDL_GL_BUFFER_SIZE, SDL_GL_DOUBLEBUFFER, SDL_GL_DEPTH_SIZE, SDL_GL_STENCIL_SIZE, \
    SDL_GL_ACCUM_RED_SIZE, SDL_GL_ACCUM_GREEN_SIZE, SDL_GL_ACCUM_BLUE_SIZE, SDL_GL_ACCUM_ALPHA_SIZE, SDL_GL_STEREO, SDL_GL_MULTISAMPLEBUFFERS, SDL_GL_MULTISAMPLESAMPLES, \
        SDL_GL_ACCELERATED_VISUAL, SDL_GL_RETAINED_BACKING, SDL_GL_CONTEXT_MAJOR_VERSION, SDL_GL_CONTEXT_MINOR_VERSION, SDL_GL_CONTEXT_FLAGS, SDL_GL_CONTEXT_PROFILE_MASK, \
            SDL_GL_SHARE_WITH_CURRENT_CONTEXT, SDL_GL_FRAMEBUFFER_SRGB_CAPABLE, SDL_GL_CONTEXT_RELEASE_BEHAVIOR, SDL_GL_CONTEXT_RESET_NOTIFICATION, \
                SDL_GL_CONTEXT_NO_ERROR, SDL_GL_FLOATBUFFERS, SDL_GL_EGL_PLATFORM = range(28)

SDL_GLProfile: typing.TypeAlias = SDL_TYPE["SDL_GLProfile", ctypes.c_uint32]

SDL_GL_CONTEXT_PROFILE_CORE: int = 0x0001
SDL_GL_CONTEXT_PROFILE_COMPATIBILITY: int = 0x0002
SDL_GL_CONTEXT_PROFILE_ES: int = 0x0004

SDL_GLContextFlag: typing.TypeAlias = SDL_TYPE["SDL_GLContextFlag", ctypes.c_uint32]

SDL_GL_CONTEXT_DEBUG_FLAG: int = 0x0001
SDL_GL_CONTEXT_FORWARD_COMPATIBLE_FLAG: int = 0x0002
SDL_GL_CONTEXT_ROBUST_ACCESS_FLAG: int = 0x0004
SDL_GL_CONTEXT_RESET_ISOLATION_FLAG: int = 0x0008

SDL_GLContextReleaseFlag: typing.TypeAlias = SDL_TYPE["SDL_GLContextReleaseFlag", ctypes.c_uint32]

SDL_GL_CONTEXT_RELEASE_BEHAVIOR_NONE: int = 0x0000
SDL_GL_CONTEXT_RELEASE_BEHAVIOR_FLUSH: int = 0x0001

SDL_GLContextResetNotification: typing.TypeAlias = SDL_TYPE["SDL_GLContextResetNotification", ctypes.c_uint32]

SDL_GL_CONTEXT_RESET_NO_NOTIFICATION: int = 0x0000
SDL_GL_CONTEXT_RESET_LOSE_CONTEXT: int = 0x0001

SDL_GetNumVideoDrivers: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetNumVideoDrivers", ctypes.c_int, [], SDL_BINARY]
SDL_GetVideoDriver: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetVideoDriver", ctypes.c_char_p, [ctypes.c_int], SDL_BINARY]
SDL_GetCurrentVideoDriver: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetCurrentVideoDriver", ctypes.c_char_p, [], SDL_BINARY]
SDL_GetSystemTheme: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetSystemTheme", SDL_SystemTheme, [], SDL_BINARY]
SDL_GetDisplays: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDisplays", SDL_POINTER[SDL_DisplayID], [SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_GetPrimaryDisplay: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetPrimaryDisplay", SDL_DisplayID, [], SDL_BINARY]
SDL_GetDisplayProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDisplayProperties", SDL_PropertiesID, [SDL_DisplayID], SDL_BINARY]

SDL_PROP_DISPLAY_HDR_ENABLED_BOOLEAN: bytes = "SDL.display.HDR_enabled".encode()
SDL_PROP_DISPLAY_KMSDRM_PANEL_ORIENTATION_NUMBER: bytes = "SDL.display.KMSDRM.panel_orientation".encode()

SDL_GetDisplayName: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDisplayName", ctypes.c_char_p, [SDL_DisplayID], SDL_BINARY]
SDL_GetDisplayBounds: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDisplayBounds", ctypes.c_bool, [SDL_DisplayID, SDL_POINTER[SDL_Rect]], SDL_BINARY]
SDL_GetDisplayUsableBounds: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDisplayUsableBounds", ctypes.c_bool, [SDL_DisplayID, SDL_POINTER[SDL_Rect]], SDL_BINARY]

SDL_GetNaturalDisplayOrientation: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetNaturalDisplayOrientation", SDL_DisplayOrientation, [SDL_DisplayID], SDL_BINARY]
SDL_GetCurrentDisplayOrientation: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetCurrentDisplayOrientation", SDL_DisplayOrientation, [SDL_DisplayID], SDL_BINARY]

SDL_GetDisplayContentScale: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDisplayContentScale", ctypes.c_float, [SDL_DisplayID], SDL_BINARY]

SDL_GetFullscreenDisplayModes: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetFullscreenDisplayModes", SDL_POINTER[SDL_POINTER[SDL_DisplayMode]], [SDL_DisplayID, SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_GetClosestFullscreenDisplayMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetClosestFullscreenDisplayMode", ctypes.c_bool, [SDL_DisplayID, ctypes.c_int, ctypes.c_int, ctypes.c_float, ctypes.c_bool, SDL_POINTER[SDL_DisplayMode]], SDL_BINARY]

SDL_GetDesktopDisplayMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDesktopDisplayMode", SDL_POINTER[SDL_DisplayMode], [SDL_DisplayID], SDL_BINARY]
SDL_GetCurrentDisplayMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetCurrentDisplayMode", SDL_POINTER[SDL_DisplayMode], [SDL_DisplayID], SDL_BINARY]

SDL_GetDisplayForPoint: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDisplayForPoint", SDL_DisplayID, [SDL_POINTER[SDL_Point]], SDL_BINARY]
SDL_GetDisplayForRect: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDisplayForRect", SDL_DisplayID, [SDL_POINTER[SDL_Rect]], SDL_BINARY]
SDL_GetDisplayForWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetDisplayForWindow", SDL_DisplayID, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_GetWindowPixelDensity: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowPixelDensity", ctypes.c_float, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_GetWindowDisplayScale: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowDisplayScale", ctypes.c_float, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_SetWindowFullscreenMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowFullscreenMode", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[SDL_DisplayMode]], SDL_BINARY]
SDL_GetWindowFullscreenMode: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowFullscreenMode", SDL_POINTER[SDL_DisplayMode], [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_GetWindowICCProfile: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowICCProfile", ctypes.c_void_p, [SDL_POINTER[SDL_Window], SDL_POINTER[ctypes.c_size_t]], SDL_BINARY]
SDL_GetWindowPixelFormat: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowPixelFormat", SDL_PixelFormat, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_GetWindows: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindows", SDL_POINTER[SDL_POINTER[SDL_Window]], [SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_CreateWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateWindow", SDL_POINTER[SDL_Window], [ctypes.c_char_p, ctypes.c_int, ctypes.c_int, SDL_WindowFlags], SDL_BINARY]
SDL_CreatePopupWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreatePopupWindow", SDL_POINTER[SDL_Window], [SDL_POINTER[SDL_Window], ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.c_int, SDL_WindowFlags], SDL_BINARY]
SDL_CreateWindowWithProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateWindowWithProperties", SDL_POINTER[SDL_Window], [SDL_PropertiesID], SDL_BINARY]

SDL_PROP_WINDOW_CREATE_ALWAYS_ON_TOP_BOOLEAN: bytes = "SDL.window.create.always_on_top".encode()
SDL_PROP_WINDOW_CREATE_BORDERLESS_BOOLEAN: bytes = "SDL.window.create.borderless".encode()
SDL_PROP_WINDOW_CREATE_FOCUSABLE_BOOLEAN: bytes = "SDL.window.create.focusable".encode()
SDL_PROP_WINDOW_CREATE_EXTERNAL_GRAPHICS_CONTEXT_BOOLEAN: bytes = "SDL.window.create.external_graphics_context".encode()
SDL_PROP_WINDOW_CREATE_FLAGS_NUMBER: bytes = "SDL.window.create.flags".encode()
SDL_PROP_WINDOW_CREATE_FULLSCREEN_BOOLEAN: bytes = "SDL.window.create.fullscreen".encode()
SDL_PROP_WINDOW_CREATE_HEIGHT_NUMBER: bytes = "SDL.window.create.height".encode()
SDL_PROP_WINDOW_CREATE_HIDDEN_BOOLEAN: bytes = "SDL.window.create.hidden".encode()
SDL_PROP_WINDOW_CREATE_HIGH_PIXEL_DENSITY_BOOLEAN: bytes = "SDL.window.create.high_pixel_density".encode()
SDL_PROP_WINDOW_CREATE_MAXIMIZED_BOOLEAN: bytes = "SDL.window.create.maximized".encode()
SDL_PROP_WINDOW_CREATE_MENU_BOOLEAN: bytes = "SDL.window.create.menu".encode()
SDL_PROP_WINDOW_CREATE_METAL_BOOLEAN: bytes = "SDL.window.create.metal".encode()
SDL_PROP_WINDOW_CREATE_MINIMIZED_BOOLEAN: bytes = "SDL.window.create.minimized".encode()
SDL_PROP_WINDOW_CREATE_MODAL_BOOLEAN: bytes = "SDL.window.create.modal".encode()
SDL_PROP_WINDOW_CREATE_MOUSE_GRABBED_BOOLEAN: bytes = "SDL.window.create.mouse_grabbed".encode()
SDL_PROP_WINDOW_CREATE_OPENGL_BOOLEAN: bytes = "SDL.window.create.opengl".encode()
SDL_PROP_WINDOW_CREATE_PARENT_POINTER: bytes = "SDL.window.create.parent".encode()
SDL_PROP_WINDOW_CREATE_RESIZABLE_BOOLEAN: bytes = "SDL.window.create.resizable".encode()
SDL_PROP_WINDOW_CREATE_TITLE_STRING: bytes = "SDL.window.create.title".encode()
SDL_PROP_WINDOW_CREATE_TRANSPARENT_BOOLEAN: bytes = "SDL.window.create.transparent".encode()
SDL_PROP_WINDOW_CREATE_TOOLTIP_BOOLEAN: bytes = "SDL.window.create.tooltip".encode()
SDL_PROP_WINDOW_CREATE_UTILITY_BOOLEAN: bytes = "SDL.window.create.utility".encode()
SDL_PROP_WINDOW_CREATE_VULKAN_BOOLEAN: bytes = "SDL.window.create.vulkan".encode()
SDL_PROP_WINDOW_CREATE_WIDTH_NUMBER: bytes = "SDL.window.create.width".encode()
SDL_PROP_WINDOW_CREATE_X_NUMBER: bytes = "SDL.window.create.x".encode()
SDL_PROP_WINDOW_CREATE_Y_NUMBER: bytes = "SDL.window.create.y".encode()
SDL_PROP_WINDOW_CREATE_COCOA_WINDOW_POINTER: bytes = "SDL.window.create.cocoa.window".encode()
SDL_PROP_WINDOW_CREATE_COCOA_VIEW_POINTER: bytes = "SDL.window.create.cocoa.view".encode()
SDL_PROP_WINDOW_CREATE_WAYLAND_SURFACE_ROLE_CUSTOM_BOOLEAN: bytes = "SDL.window.create.wayland.surface_role_custom".encode()
SDL_PROP_WINDOW_CREATE_WAYLAND_CREATE_EGL_WINDOW_BOOLEAN: bytes = "SDL.window.create.wayland.create_egl_window".encode()
SDL_PROP_WINDOW_CREATE_WAYLAND_WL_SURFACE_POINTER: bytes = "SDL.window.create.wayland.wl_surface".encode()
SDL_PROP_WINDOW_CREATE_WIN32_HWND_POINTER: bytes = "SDL.window.create.win32.hwnd".encode()
SDL_PROP_WINDOW_CREATE_WIN32_PIXEL_FORMAT_HWND_POINTER: bytes = "SDL.window.create.win32.pixel_format_hwnd".encode()
SDL_PROP_WINDOW_CREATE_X11_WINDOW_NUMBER: bytes = "SDL.window.create.x11.window".encode()

SDL_GetWindowID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowID", SDL_WindowID, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_GetWindowFromID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowFromID", SDL_POINTER[SDL_Window], [SDL_WindowID], SDL_BINARY]
SDL_GetWindowParent: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowParent", SDL_POINTER[SDL_Window], [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_GetWindowProperties: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowProperties", SDL_PropertiesID, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_PROP_WINDOW_SHAPE_POINTER: bytes = "SDL.window.shape".encode()
SDL_PROP_WINDOW_HDR_ENABLED_BOOLEAN: bytes = "SDL.window.HDR_enabled".encode()
SDL_PROP_WINDOW_SDR_WHITE_LEVEL_FLOAT: bytes = "SDL.window.SDR_white_level".encode()
SDL_PROP_WINDOW_HDR_HEADROOM_FLOAT: bytes = "SDL.window.HDR_headroom".encode()
SDL_PROP_WINDOW_ANDROID_WINDOW_POINTER: bytes = "SDL.window.android.window".encode()
SDL_PROP_WINDOW_ANDROID_SURFACE_POINTER: bytes = "SDL.window.android.surface".encode()
SDL_PROP_WINDOW_UIKIT_WINDOW_POINTER: bytes = "SDL.window.uikit.window".encode()
SDL_PROP_WINDOW_UIKIT_METAL_VIEW_TAG_NUMBER: bytes = "SDL.window.uikit.metal_view_tag".encode()
SDL_PROP_WINDOW_UIKIT_OPENGL_FRAMEBUFFER_NUMBER: bytes = "SDL.window.uikit.opengl.framebuffer".encode()
SDL_PROP_WINDOW_UIKIT_OPENGL_RENDERBUFFER_NUMBER: bytes = "SDL.window.uikit.opengl.renderbuffer".encode()
SDL_PROP_WINDOW_UIKIT_OPENGL_RESOLVE_FRAMEBUFFER_NUMBER: bytes = "SDL.window.uikit.opengl.resolve_framebuffer".encode()
SDL_PROP_WINDOW_KMSDRM_DEVICE_INDEX_NUMBER: bytes = "SDL.window.kmsdrm.dev_index".encode()
SDL_PROP_WINDOW_KMSDRM_DRM_FD_NUMBER: bytes = "SDL.window.kmsdrm.drm_fd".encode()
SDL_PROP_WINDOW_KMSDRM_GBM_DEVICE_POINTER: bytes = "SDL.window.kmsdrm.gbm_dev".encode()
SDL_PROP_WINDOW_COCOA_WINDOW_POINTER: bytes = "SDL.window.cocoa.window".encode()
SDL_PROP_WINDOW_COCOA_METAL_VIEW_TAG_NUMBER: bytes = "SDL.window.cocoa.metal_view_tag".encode()
SDL_PROP_WINDOW_OPENVR_OVERLAY_ID: bytes = "SDL.window.openvr.overlay_id".encode()
SDL_PROP_WINDOW_VIVANTE_DISPLAY_POINTER: bytes = "SDL.window.vivante.display".encode()
SDL_PROP_WINDOW_VIVANTE_WINDOW_POINTER: bytes = "SDL.window.vivante.window".encode()
SDL_PROP_WINDOW_VIVANTE_SURFACE_POINTER: bytes = "SDL.window.vivante.surface".encode()
SDL_PROP_WINDOW_WIN32_HWND_POINTER: bytes = "SDL.window.win32.hwnd".encode()
SDL_PROP_WINDOW_WIN32_HDC_POINTER: bytes = "SDL.window.win32.hdc".encode()
SDL_PROP_WINDOW_WIN32_INSTANCE_POINTER: bytes = "SDL.window.win32.instance".encode()
SDL_PROP_WINDOW_WAYLAND_DISPLAY_POINTER: bytes = "SDL.window.wayland.display".encode()
SDL_PROP_WINDOW_WAYLAND_SURFACE_POINTER: bytes = "SDL.window.wayland.surface".encode()
SDL_PROP_WINDOW_WAYLAND_VIEWPORT_POINTER: bytes = "SDL.window.wayland.viewport".encode()
SDL_PROP_WINDOW_WAYLAND_EGL_WINDOW_POINTER: bytes = "SDL.window.wayland.egl_window".encode()
SDL_PROP_WINDOW_WAYLAND_XDG_SURFACE_POINTER: bytes = "SDL.window.wayland.xdg_surface".encode()
SDL_PROP_WINDOW_WAYLAND_XDG_TOPLEVEL_POINTER: bytes = "SDL.window.wayland.xdg_toplevel".encode()
SDL_PROP_WINDOW_WAYLAND_XDG_TOPLEVEL_EXPORT_HANDLE_STRING: bytes = "SDL.window.wayland.xdg_toplevel_export_handle".encode()
SDL_PROP_WINDOW_WAYLAND_XDG_POPUP_POINTER: bytes = "SDL.window.wayland.xdg_popup".encode()
SDL_PROP_WINDOW_WAYLAND_XDG_POSITIONER_POINTER: bytes = "SDL.window.wayland.xdg_positioner".encode()
SDL_PROP_WINDOW_X11_DISPLAY_POINTER: bytes = "SDL.window.x11.display".encode()
SDL_PROP_WINDOW_X11_SCREEN_NUMBER: bytes = "SDL.window.x11.screen".encode()
SDL_PROP_WINDOW_X11_WINDOW_NUMBER: bytes = "SDL.window.x11.window".encode()

SDL_GetWindowFlags: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowFlags", SDL_WindowFlags, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_SetWindowTitle: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowTitle", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_char_p], SDL_BINARY]
SDL_GetWindowTitle: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowTitle", ctypes.c_char_p, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_SetWindowIcon: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowIcon", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[SDL_Surface]], SDL_BINARY]

SDL_SetWindowPosition: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowPosition", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_int, ctypes.c_int], SDL_BINARY]
SDL_GetWindowPosition: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowPosition", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_SetWindowSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowSize", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_int, ctypes.c_int], SDL_BINARY]
SDL_GetWindowSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowSize", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_GetWindowSafeArea: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowSafeArea", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[SDL_Rect]], SDL_BINARY]

SDL_SetWindowAspectRatio: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowAspectRatio", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_float, ctypes.c_float], SDL_BINARY]
SDL_GetWindowAspectRatio: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowAspectRatio", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[ctypes.c_float], SDL_POINTER[ctypes.c_float]], SDL_BINARY]

SDL_GetWindowBordersSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowBordersSize", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_GetWindowSizeInPixels: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowSizeInPixels", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_SetWindowMinimumSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowMinimumSize", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_int, ctypes.c_int], SDL_BINARY]
SDL_GetWindowMinimumSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowMinimumSize", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_SetWindowMaximumSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowMaximumSize", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_int, ctypes.c_int], SDL_BINARY]
SDL_GetWindowMaximumSize: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowMaximumSize", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[ctypes.c_int], SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_SetWindowBordered: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowBordered", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_bool], SDL_BINARY]
SDL_SetWindowResizable: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowResizable", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_bool], SDL_BINARY]
SDL_SetWindowAlwaysOnTop: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowAlwaysOnTop", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_bool], SDL_BINARY]

SDL_ShowWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShowWindow", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_HideWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_HideWindow", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_RaiseWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RaiseWindow", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_MaximizeWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_MaximizeWindow", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_MinimizeWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_MinimizeWindow", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_RestoreWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_RestoreWindow", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_SetWindowFullscreen: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowFullscreen", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_bool], SDL_BINARY]
SDL_SyncWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SyncWindow", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_WindowHasSurface: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WindowHasSurface", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_GetWindowSurface: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowSurface", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_SetWindowSurfaceVSync: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowSurfaceVSync", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_int], SDL_BINARY]

SDL_WINDOW_SURFACE_VSYNC_DISABLED, SDL_WINDOW_SURFACE_VSYNC_ADAPTIVE = 0, -1

SDL_GetWindowSurfaceVSync: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowSurfaceVSync", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_UpdateWindowSurface: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UpdateWindowSurface", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_UpdateWindowSurfaceRects: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_UpdateWindowSurfaceRects", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[SDL_Rect], ctypes.c_int], SDL_BINARY]
SDL_DestroyWindowSurface: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_DestroyWindowSurface", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_SetWindowKeyboardGrab: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowKeyboardGrab", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_bool], SDL_BINARY]
SDL_SetWindowMouseGrab: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowMouseGrab", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_bool], SDL_BINARY]

SDL_GetWindowKeyboardGrab: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowKeyboardGrab", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_GetWindowMouseGrab: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowMouseGrab", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_GetGrabbedWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetGrabbedWindow", SDL_POINTER[SDL_Window], [], SDL_BINARY]

SDL_SetWindowMouseRect: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowMouseRect", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[SDL_Rect]], SDL_BINARY]
SDL_GetWindowMouseRect: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowMouseRect", SDL_POINTER[SDL_Rect], [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_SetWindowOpacity: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowOpacity", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_float], SDL_BINARY]
SDL_GetWindowOpacity: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetWindowOpacity", ctypes.c_float, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_SetWindowParent: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowParent", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_SetWindowModal: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowModal", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_bool], SDL_BINARY]
SDL_SetWindowFocusable: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowFocusable", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_bool], SDL_BINARY]

SDL_ShowWindowSystemMenu: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShowWindowSystemMenu", ctypes.c_bool, [SDL_POINTER[SDL_Window], ctypes.c_int, ctypes.c_int], SDL_BINARY]

SDL_HitTestResult: typing.TypeAlias = SDL_TYPE["SDL_HitTestResult", SDL_ENUM]

SDL_HITTEST_NORMAL, SDL_HITTEST_DRAGGABLE, SDL_HITTEST_RESIZE_TOPLEFT, SDL_HITTEST_RESIZE_TOP, SDL_HITTEST_RESIZE_TOPRIGHT, \
    SDL_HITTEST_RESIZE_RIGHT, SDL_HITTEST_RESIZE_BOTTOMRIGHT, SDL_HITTEST_RESIZE_BOTTOM, SDL_HITTEST_RESIZE_BOTTOMLEFT, SDL_HITTEST_RESIZE_LEFT = range(10)

SDL_HitTest: typing.TypeAlias = SDL_FUNC_TYPE["SDL_HitTest", SDL_HitTestResult, [SDL_POINTER[SDL_Window], SDL_POINTER[SDL_Point], ctypes.c_void_p]]

SDL_SetWindowHitTest: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowHitTest", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_HitTest, ctypes.c_void_p], SDL_BINARY]
SDL_SetWindowShape: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetWindowShape", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_POINTER[SDL_Surface]], SDL_BINARY]

SDL_FlashWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_FlashWindow", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_FlashOperation], SDL_BINARY]
SDL_DestroyWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_DestroyWindow", None, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_ScreenSaverEnabled: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ScreenSaverEnabled", ctypes.c_bool, [], SDL_BINARY]
SDL_EnableScreenSaver: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_EnableScreenSaver", ctypes.c_bool, [], SDL_BINARY]
SDL_DisableScreenSaver: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_DisableScreenSaver", ctypes.c_bool, [], SDL_BINARY]

SDL_GL_LoadLibrary: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_LoadLibrary", ctypes.c_bool, [ctypes.c_char_p], SDL_BINARY]
SDL_GL_GetProcAddress: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_GetProcAddress", SDL_FunctionPointer, [ctypes.c_char_p], SDL_BINARY]
SDL_EGL_GetProcAddress: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_EGL_GetProcAddress", SDL_FunctionPointer, [ctypes.c_char_p], SDL_BINARY]
SDL_GL_UnloadLibrary: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_UnloadLibrary", None, [], SDL_BINARY]

SDL_GL_ExtensionSupported: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_ExtensionSupported", ctypes.c_bool, [ctypes.c_char_p], SDL_BINARY]

SDL_GL_ResetAttributes: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_ResetAttributes", None, [], SDL_BINARY]
SDL_GL_SetAttribute: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_SetAttribute", ctypes.c_bool, [SDL_GLAttr, ctypes.c_int], SDL_BINARY]
SDL_GL_GetAttribute: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_GetAttribute", ctypes.c_bool, [SDL_GLAttr, SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_GL_CreateContext: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_CreateContext", SDL_GLContext, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_GL_MakeCurrent: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_MakeCurrent", ctypes.c_bool, [SDL_POINTER[SDL_Window], SDL_GLContext], SDL_BINARY]

SDL_GL_GetCurrentWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_GetCurrentWindow", SDL_POINTER[SDL_Window], [], SDL_BINARY]
SDL_GL_GetCurrentContext: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_GetCurrentContext", SDL_GLContext, [], SDL_BINARY]

SDL_EGL_GetCurrentDisplay: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_EGL_GetCurrentDisplay", SDL_EGLDisplay, [], SDL_BINARY]
SDL_EGL_GetCurrentConfig: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_EGL_GetCurrentConfig", SDL_EGLConfig, [], SDL_BINARY]
SDL_EGL_GetWindowSurface: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_EGL_GetWindowSurface", SDL_EGLSurface, [SDL_POINTER[SDL_Window]], SDL_BINARY]

SDL_EGL_SetAttributeCallbacks: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_EGL_SetAttributeCallbacks", None, [SDL_EGLAttribArrayCallback, SDL_EGLIntArrayCallback, SDL_EGLIntArrayCallback, ctypes.c_void_p], SDL_BINARY]

SDL_GL_SetSwapInterval: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_SetSwapInterval", ctypes.c_bool, [ctypes.c_int], SDL_BINARY]
SDL_GL_GetSwapInterval: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_GetSwapInterval", ctypes.c_bool, [SDL_POINTER[ctypes.c_int]], SDL_BINARY]

SDL_GL_SwapWindow: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_SwapWindow", ctypes.c_bool, [SDL_POINTER[SDL_Window]], SDL_BINARY]
SDL_GL_DestroyContext: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GL_DestroyContext", ctypes.c_bool, [SDL_GLContext], SDL_BINARY]


from .__init__ import ctypes, typing, abc, \
    SDL_POINTER, SDL_FUNC, SDL_IMAGE_BINARY

from .SDL_surface import SDL_Surface
from .SDL_render import SDL_Texture, SDL_Renderer
from .SDL_iostream import SDL_IOStream
from .SDL_version import SDL_VERSIONNUM

SDL_IMAGE_MAJOR_VERSION, SDL_IMAGE_MINOR_VERSION, SDL_IMAGE_MICRO_VERSION = 3, 2, 4
SDL_IMAGE_VERSION: int = SDL_VERSIONNUM(SDL_IMAGE_MAJOR_VERSION, SDL_IMAGE_MINOR_VERSION, SDL_IMAGE_MICRO_VERSION)

SDL_IMAGE_VERSION_ATLEAST: abc.Callable[[int, int, int], bool] = lambda x, y, z: \
    (SDL_IMAGE_MAJOR_VERSION >= x) and (SDL_IMAGE_MAJOR_VERSION > x or SDL_IMAGE_MINOR_VERSION >= y) and \
        (SDL_IMAGE_MAJOR_VERSION > x or SDL_IMAGE_MINOR_VERSION > y or SDL_IMAGE_MICRO_VERSION >= z)

IMG_Version: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_Version", ctypes.c_int, [], SDL_IMAGE_BINARY]

IMG_LoadTyped_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadTyped_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream], ctypes.c_bool, ctypes.c_char_p], SDL_IMAGE_BINARY]
IMG_Load: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_Load", SDL_POINTER[SDL_Surface], [ctypes.c_char_p], SDL_IMAGE_BINARY]
IMG_Load_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_Load_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream], ctypes.c_bool], SDL_IMAGE_BINARY]

IMG_LoadTexture: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadTexture", SDL_POINTER[SDL_Texture], [SDL_POINTER[SDL_Renderer], ctypes.c_char_p], SDL_IMAGE_BINARY]
IMG_LoadTexture_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadTexture_IO", SDL_POINTER[SDL_Texture], [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_IOStream], ctypes.c_bool], SDL_IMAGE_BINARY]
IMG_LoadTextureTyped_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadTextureTyped_IO", SDL_POINTER[SDL_Texture], [SDL_POINTER[SDL_Renderer], SDL_POINTER[SDL_IOStream], ctypes.c_bool, ctypes.c_char_p], SDL_IMAGE_BINARY]

IMG_isAVIF: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isAVIF", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isICO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isICO", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isCUR: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isCUR", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isBMP: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isBMP", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isGIF: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isGIF", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isJPG: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isJPG", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isJXL: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isJXL", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isLBM: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isLBM", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isPCX: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isPCX", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isPNG: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isPNG", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isPNM: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isPNM", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isSVG: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isSVG", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isQOI: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isQOI", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isTIF: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isTIF", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isXCF: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isXCF", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isXPM: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isXPM", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isXV: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isXV", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_isWEBP: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_isWEBP", ctypes.c_bool, [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]

IMG_LoadAVIF_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadAVIF_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadICO_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadICO_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadCUR_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadCUR_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadBMP_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadBMP_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadGIF_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadGIF_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadJPG_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadJPG_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadJXL_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadJXL_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadLBM_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadLBM_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadPCX_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadPCX_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadPNG_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadPNG_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadPNM_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadPNM_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadSVG_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadSVG_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadQOI_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadQOI_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadTGA_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadTGA_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadTIF_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadTIF_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadXCF_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadXCF_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadXPM_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadXPM_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadXV_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadXV_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadWEBP_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadWEBP_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadSizedSVG_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadSizedSVG_IO", SDL_POINTER[SDL_Surface], [SDL_POINTER[SDL_IOStream], ctypes.c_int, ctypes.c_int], SDL_IMAGE_BINARY]

IMG_ReadXPMFromArray: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_ReadXPMFromArray", SDL_POINTER[SDL_Surface], [SDL_POINTER[ctypes.c_char_p]], SDL_IMAGE_BINARY]
IMG_ReadXPMFromArrayToRGB888: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_ReadXPMFromArrayToRGB888", SDL_POINTER[SDL_Surface], [SDL_POINTER[ctypes.c_char_p]], SDL_IMAGE_BINARY]

IMG_SaveAVIF: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_SaveAVIF", ctypes.c_bool, [SDL_POINTER[SDL_Surface], ctypes.c_char_p, ctypes.c_int], SDL_IMAGE_BINARY]
IMG_SaveAVIF_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_SaveAVIF_IO", ctypes.c_bool, [SDL_POINTER[SDL_Surface], SDL_POINTER[SDL_IOStream], ctypes.c_bool, ctypes.c_int], SDL_IMAGE_BINARY]
IMG_SavePNG: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_SavePNG", ctypes.c_bool, [SDL_POINTER[SDL_Surface], ctypes.c_char_p], SDL_IMAGE_BINARY]
IMG_SavePNG_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_SavePNG_IO", ctypes.c_bool, [SDL_POINTER[SDL_Surface], SDL_POINTER[SDL_IOStream], ctypes.c_bool], SDL_IMAGE_BINARY]
IMG_SaveJPG: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_SaveJPG", ctypes.c_bool, [SDL_POINTER[SDL_Surface], ctypes.c_char_p, ctypes.c_int], SDL_IMAGE_BINARY]
IMG_SaveJPG_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_SaveJPG_IO", ctypes.c_bool, [SDL_POINTER[SDL_Surface], SDL_POINTER[SDL_IOStream], ctypes.c_bool, ctypes.c_int], SDL_IMAGE_BINARY]

class IMG_Animation(ctypes.Structure):
    _fields_ = [
        ("w", ctypes.c_int),
        ("h", ctypes.c_int),
        ("count", ctypes.c_int),
        ("frames", SDL_POINTER[SDL_POINTER[SDL_Surface]]),
        ("delays", SDL_POINTER[ctypes.c_int])
    ]

IMG_LoadAnimation: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadAnimation", SDL_POINTER[IMG_Animation], [ctypes.c_char_p], SDL_IMAGE_BINARY]
IMG_LoadAnimation_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadAnimation_IO", SDL_POINTER[IMG_Animation], [SDL_POINTER[SDL_IOStream], ctypes.c_bool], SDL_IMAGE_BINARY]
IMG_LoadAnimationTyped_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadAnimationTyped_IO", SDL_POINTER[IMG_Animation], [SDL_POINTER[SDL_IOStream], ctypes.c_bool, ctypes.c_char_p], SDL_IMAGE_BINARY]
IMG_FreeAnimation: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_FreeAnimation", None, [SDL_POINTER[IMG_Animation]], SDL_IMAGE_BINARY]
IMG_LoadGIFAnimation_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadGIFAnimation_IO", SDL_POINTER[IMG_Animation], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]
IMG_LoadWEBPAnimation_IO: abc.Callable[..., typing.Any] = SDL_FUNC["IMG_LoadWEBPAnimation_IO", SDL_POINTER[IMG_Animation], [SDL_POINTER[SDL_IOStream]], SDL_IMAGE_BINARY]


from .__init__ import ctypes, typing, abc, SDL_POINTER, \
    SDL_FUNC, SDL_TYPE, SDL_ENUM, SDL_SHADERCROSS_BINARY

from .SDL_properties import SDL_PropertiesID
from .SDL_gpu import SDL_GPUDevice, SDL_GPUShaderFormat, SDL_GPUShader, SDL_GPUComputePipeline

SDL_SHADERCROSS_MAJOR_VERSION, SDL_SHADERCROSS_MINOR_VERSION, SDL_SHADERCROSS_MICRO_VERSION = 3, 0, 0

SDL_ShaderCross_IOVarType: typing.TypeAlias = SDL_TYPE["SDL_ShaderCross_IOVarType", SDL_ENUM]

SDL_SHADERCROSS_IOVAR_TYPE_UNKNOWN, SDL_SHADERCROSS_IOVAR_TYPE_INT8, SDL_SHADERCROSS_IOVAR_TYPE_UINT8, SDL_SHADERCROSS_IOVAR_TYPE_INT16, SDL_SHADERCROSS_IOVAR_TYPE_UINT16, \
    SDL_SHADERCROSS_IOVAR_TYPE_INT32, SDL_SHADERCROSS_IOVAR_TYPE_UINT32, SDL_SHADERCROSS_IOVAR_TYPE_INT64, SDL_SHADERCROSS_IOVAR_TYPE_UINT64, \
        SDL_SHADERCROSS_IOVAR_TYPE_FLOAT16, SDL_SHADERCROSS_IOVAR_TYPE_FLOAT32, SDL_SHADERCROSS_IOVAR_TYPE_FLOAT64 = range(12)

SDL_ShaderCross_ShaderStage: typing.TypeAlias = SDL_TYPE["SDL_ShaderCross_ShaderStage", SDL_ENUM]

SDL_SHADERCROSS_SHADERSTAGE_VERTEX, SDL_SHADERCROSS_SHADERSTAGE_FRAGMENT, SDL_SHADERCROSS_SHADERSTAGE_COMPUTE = range(3)

class SDL_ShaderCross_IOVarMetadata(ctypes.Structure):
    _fields_ = [
        ("name", ctypes.c_char_p),
        ("location", ctypes.c_uint32),
        ("offset", ctypes.c_uint32),
        ("vector_type", SDL_ShaderCross_IOVarType),
        ("vector_size", ctypes.c_uint32)
    ]

class SDL_ShaderCross_GraphicsShaderMetadata(ctypes.Structure):
    _fields_ = [
        ("num_samplers", ctypes.c_uint32),
        ("num_storage_textures", ctypes.c_uint32),
        ("num_storage_buffers", ctypes.c_uint32),
        ("num_uniform_buffers", ctypes.c_uint32),
        ("num_inputs", ctypes.c_uint32),
        ("inputs", SDL_POINTER[SDL_ShaderCross_IOVarMetadata]),
        ("num_outputs", ctypes.c_uint32),
        ("outputs", SDL_POINTER[SDL_ShaderCross_IOVarMetadata])
    ]

class SDL_ShaderCross_ComputePipelineMetadata(ctypes.Structure):
    _fields_ = [
        ("num_samplers", ctypes.c_uint32),
        ("num_readonly_storage_textures", ctypes.c_uint32),
        ("num_readonly_storage_buffers", ctypes.c_uint32),
        ("num_readwrite_storage_textures", ctypes.c_uint32),
        ("num_readwrite_storage_buffers", ctypes.c_uint32),
        ("num_uniform_buffers", ctypes.c_uint32),
        ("threadcount_x", ctypes.c_uint32),
        ("threadcount_y", ctypes.c_uint32),
        ("threadcount_z", ctypes.c_uint32)
    ]

class SDL_ShaderCross_SPIRV_Info(ctypes.Structure):
    _fields_ = [
        ("bytecode", SDL_POINTER[ctypes.c_uint8]),
        ("bytecode_size", ctypes.c_size_t),
        ("entrypoint", ctypes.c_char_p),
        ("shader_stage", SDL_ShaderCross_ShaderStage),
        ("enable_debug", ctypes.c_bool),
        ("name", ctypes.c_char_p),
        ("props", SDL_PropertiesID)
    ]

SDL_SHADERCROSS_PROP_SPIRV_PSSL_COMPATIBILITY: bytes = "SDL.shadercross.spirv.pssl.compatibility".encode()
SDL_SHADERCROSS_PROP_SPIRV_MSL_VERSION: bytes = "SDL.shadercross.spirv.msl.version".encode()

class SDL_ShaderCross_HLSL_Define(ctypes.Structure):
    _fields_ = [
        ("name", ctypes.c_char_p),
        ("value", ctypes.c_char_p)
    ]

class SDL_ShaderCross_HLSL_Info(ctypes.Structure):
    _fields_ = [
        ("source", ctypes.c_char_p),
        ("entrypoint", ctypes.c_char_p),
        ("include_dir", ctypes.c_char_p),
        ("defines", SDL_POINTER[SDL_ShaderCross_HLSL_Define]),
        ("shader_stage", SDL_ShaderCross_ShaderStage),
        ("enable_debug", ctypes.c_bool),
        ("name", ctypes.c_char_p),
        ("props", SDL_PropertiesID)
    ]

SDL_ShaderCross_Init: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_Init", ctypes.c_bool, [], SDL_SHADERCROSS_BINARY]
SDL_ShaderCross_Quit: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_Quit", None, [], SDL_SHADERCROSS_BINARY]

SDL_ShaderCross_GetSPIRVShaderFormats: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_GetSPIRVShaderFormats", SDL_GPUShaderFormat, [], SDL_SHADERCROSS_BINARY]

SDL_ShaderCross_TranspileMSLFromSPIRV: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_TranspileMSLFromSPIRV", ctypes.c_void_p, [SDL_POINTER[SDL_ShaderCross_SPIRV_Info]], SDL_SHADERCROSS_BINARY]
SDL_ShaderCross_TranspileHLSLFromSPIRV: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_TranspileHLSLFromSPIRV", ctypes.c_void_p, [SDL_POINTER[SDL_ShaderCross_SPIRV_Info]], SDL_SHADERCROSS_BINARY]
SDL_ShaderCross_CompileDXBCFromSPIRV: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_CompileDXBCFromSPIRV", ctypes.c_void_p, [SDL_POINTER[SDL_ShaderCross_SPIRV_Info], SDL_POINTER[ctypes.c_size_t]], SDL_SHADERCROSS_BINARY]
SDL_ShaderCross_CompileDXILFromSPIRV: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_CompileDXILFromSPIRV", ctypes.c_void_p, [SDL_POINTER[SDL_ShaderCross_SPIRV_Info], SDL_POINTER[ctypes.c_size_t]], SDL_SHADERCROSS_BINARY]

SDL_ShaderCross_CompileGraphicsShaderFromSPIRV: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_CompileGraphicsShaderFromSPIRV", SDL_POINTER[SDL_GPUShader], [SDL_POINTER[SDL_GPUDevice], SDL_POINTER[SDL_ShaderCross_SPIRV_Info], SDL_POINTER[SDL_ShaderCross_GraphicsShaderMetadata], SDL_PropertiesID], SDL_SHADERCROSS_BINARY]
SDL_ShaderCross_CompileComputePipelineFromSPIRV: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_CompileComputePipelineFromSPIRV", SDL_POINTER[SDL_GPUComputePipeline], [SDL_POINTER[SDL_GPUDevice], SDL_POINTER[SDL_ShaderCross_SPIRV_Info], SDL_POINTER[SDL_ShaderCross_ComputePipelineMetadata], SDL_PropertiesID], SDL_SHADERCROSS_BINARY]

SDL_ShaderCross_ReflectGraphicsSPIRV: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_ReflectGraphicsSPIRV", SDL_POINTER[SDL_ShaderCross_GraphicsShaderMetadata], [SDL_POINTER[ctypes.c_uint8], ctypes.c_size_t, SDL_PropertiesID], SDL_SHADERCROSS_BINARY]
SDL_ShaderCross_ReflectComputeSPIRV: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_ReflectComputeSPIRV", SDL_POINTER[SDL_ShaderCross_ComputePipelineMetadata], [SDL_POINTER[ctypes.c_uint8], ctypes.c_size_t, SDL_PropertiesID], SDL_SHADERCROSS_BINARY]

SDL_ShaderCross_GetHLSLShaderFormats: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_GetHLSLShaderFormats", SDL_GPUShaderFormat, [], SDL_SHADERCROSS_BINARY]

SDL_ShaderCross_CompileDXBCFromHLSL: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_CompileDXBCFromHLSL", ctypes.c_void_p, [SDL_POINTER[SDL_ShaderCross_HLSL_Info], SDL_POINTER[ctypes.c_size_t]], SDL_SHADERCROSS_BINARY]
SDL_ShaderCross_CompileDXILFromHLSL: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_CompileDXILFromHLSL", ctypes.c_void_p, [SDL_POINTER[SDL_ShaderCross_HLSL_Info], SDL_POINTER[ctypes.c_size_t]], SDL_SHADERCROSS_BINARY]
SDL_ShaderCross_CompileSPIRVFromHLSL: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_ShaderCross_CompileSPIRVFromHLSL", ctypes.c_void_p, [SDL_POINTER[SDL_ShaderCross_HLSL_Info], SDL_POINTER[ctypes.c_size_t]], SDL_SHADERCROSS_BINARY]


from .__init__ import ctypes, typing, abc, SDL_POINTER, \
    SDL_FUNC_TYPE, SDL_ENUM, SDL_FUNC, SDL_TYPE, SDL_BINARY

from .SDL_atomic import SDL_AtomicInt
from .SDL_stdinc import SDL_FunctionPointer
from .SDL_properties import SDL_PropertiesID

class SDL_Thread(ctypes.c_void_p):
    ...

SDL_ThreadID: typing.TypeAlias = SDL_TYPE["SDL_ThreadID", ctypes.c_uint64]
SDL_TLSID: typing.TypeAlias = SDL_TYPE["SDL_TLSID", SDL_AtomicInt]

SDL_ThreadPriority: typing.TypeAlias = SDL_TYPE["SDL_ThreadPriority", SDL_ENUM]

SDL_THREAD_PRIORITY_LOW, SDL_THREAD_PRIORITY_NORMAL, SDL_THREAD_PRIORITY_HIGH, SDL_THREAD_PRIORITY_TIME_CRITICAL = range(4)

SDL_ThreadState: typing.TypeAlias = SDL_TYPE["SDL_ThreadState", SDL_ENUM]

SDL_THREAD_UNKNOWN, SDL_THREAD_ALIVE, SDL_THREAD_DETACHED, SDL_THREAD_COMPLETE = range(4)

SDL_ThreadFunction: typing.TypeAlias = SDL_FUNC_TYPE["SDL_ThreadFunction", ctypes.c_int, [ctypes.c_void_p]]

SDL_BeginThreadFunction, SDL_EndThreadFunction = SDL_FunctionPointer(0), SDL_FunctionPointer(0)

SDL_CreateThreadRuntime: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateThreadRuntime", SDL_POINTER[SDL_Thread], [SDL_ThreadFunction, ctypes.c_char_p, ctypes.c_void_p, SDL_FunctionPointer, SDL_FunctionPointer], SDL_BINARY]
SDL_CreateThreadWithPropertiesRuntime: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CreateThreadWithPropertiesRuntime", SDL_POINTER[SDL_Thread], [SDL_PropertiesID, SDL_FunctionPointer, SDL_FunctionPointer], SDL_BINARY]

SDL_CreateThread: abc.Callable[[SDL_ThreadFunction, ctypes.c_char_p, ctypes.c_void_p], SDL_POINTER[SDL_Thread]] = \
    lambda fn, name, data: SDL_CreateThreadRuntime(fn, name, data, SDL_BeginThreadFunction, SDL_EndThreadFunction)

SDL_CreateThreadWithProperties: abc.Callable[[SDL_PropertiesID], SDL_POINTER[SDL_Thread]] = \
    lambda props: SDL_CreateThreadWithPropertiesRuntime(props, SDL_BeginThreadFunction, SDL_EndThreadFunction)

SDL_PROP_THREAD_CREATE_ENTRY_FUNCTION_POINTER: bytes = "SDL.thread.create.entry_function".encode()
SDL_PROP_THREAD_CREATE_NAME_STRING: bytes = "SDL.thread.create.name".encode()
SDL_PROP_THREAD_CREATE_USERDATA_POINTER: bytes = "SDL.thread.create.userdata".encode()
SDL_PROP_THREAD_CREATE_STACKSIZE_NUMBER: bytes = "SDL.thread.create.stacksize".encode()

SDL_GetThreadName: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetThreadName", ctypes.c_char_p, [SDL_POINTER[SDL_Thread]], SDL_BINARY]
SDL_GetCurrentThreadID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetCurrentThreadID", SDL_ThreadID, [], SDL_BINARY]
SDL_GetThreadID: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetThreadID", SDL_ThreadID, [SDL_POINTER[SDL_Thread]], SDL_BINARY]
SDL_SetCurrentThreadPriority: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetCurrentThreadPriority", ctypes.c_bool, [SDL_ThreadPriority], SDL_BINARY]
SDL_WaitThread: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_WaitThread", None, [SDL_POINTER[SDL_Thread], SDL_POINTER[ctypes.c_int]], SDL_BINARY]
SDL_GetThreadState: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetThreadState", SDL_ThreadState, [SDL_POINTER[SDL_Thread]], SDL_BINARY]
SDL_DetachThread: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_DetachThread", None, [SDL_POINTER[SDL_Thread]], SDL_BINARY]
SDL_GetTLS: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_GetTLS", ctypes.c_void_p, [SDL_POINTER[SDL_TLSID]], SDL_BINARY]

SDL_TLSDestructorCallback: typing.TypeAlias = SDL_FUNC_TYPE["SDL_TLSDestructorCallback", None, [ctypes.c_void_p]]

SDL_SetTLS: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_SetTLS", ctypes.c_bool, [SDL_POINTER[SDL_TLSID], ctypes.c_void_p, SDL_TLSDestructorCallback], SDL_BINARY]
SDL_CleanupTLS: abc.Callable[..., typing.Any] = SDL_FUNC["SDL_CleanupTLS", None, [], SDL_BINARY]